/*
 * SmartcardioNFC - a library providing a smartcardio implementation for NFC on Android
 *
 * Copyright (C) 2017 gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH <info@gematik.de> +49 30 40041-0
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.test;

import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ShortBuffer;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import javax.smartcardio.CardException;

import de.gematik.security.Utilities.RemoteCardChannel;
import de.gematik.security.Utilities.Utilities;
import de.gematik.security.impl.HealthcardContext;
import de.gematik.security.iso7816.Iso7816UnsuccessfulExecutionException;
import de.gematik.security.iso9564.PinBlockException;
import de.gematik.security.usbCard.UsbCard;

@RunWith(AndroidJUnit4.class)
public class Card2CardTest extends NfcBaseTest {
    private static final String TAG = Card2CardTest.class.getName();

    @Test
    public void testWithUsbCard() throws CardException, IOException, Iso7816UnsuccessfulExecutionException, PinBlockException, NoSuchProviderException, NoSuchAlgorithmException {
        Log.i(TAG, "testWithUsbCard()");

        javax.smartcardio.Card usbCard = UsbCard.easyConnect(activity, "T=1");
        HealthcardContext remoteCard = new HealthcardContext(usbCard.openLogicalChannel(), true);

        HealthcardContext egk;
        HealthcardContext hba;

        if (healthcardContext.isEGK()) {
            egk = healthcardContext;
            hba = remoteCard;
        } else {
            egk = remoteCard;
            hba = healthcardContext;
        }

        // test Card2Card authentication:

        // Use Case 1: Freischalten einer EGK durch einen HBA, z.B. zum Zugriff auf die Notfalldaten (NFD)
        // precondition: PIN.CH/PIN.SMC verified for HBA/SMC-B
        hba.verify("123456");
        egk.authenticateOtherCard(hba);

        // Use Case 2: Echtheitsprüfung einer EGK durch einen HBA
        hba.authenticateOtherCard(egk);

        // test access to NFD in order to verify the success of Card2Card authentication:

        //SELECT DF.HCA
        egk.selectADF("D27600000102");

        //SELECT DF.HCA/DF.NFD
        egk.selectADF("D27600014407");

        //SELECT EF.NFD
        egk.selectEF("D010");

        if (egk.isExtendedLengthApduSupported()) {
            egk.setMaxLength(4096); // readBinary funktioniert nur bis LE=4096, LE=8192 und größer liefert den Error 6F00
        }

        // Read length
        ShortBuffer shortBuffer = ByteBuffer.wrap(egk.read(0, 2)).asShortBuffer();

        // Read data
        byte[] data = egk.read(2, shortBuffer.get(0));
        String result = Utilities.unzip(data);
        Log.i(TAG, result);
    }

    @Test
    public void testWithRemoteCardServer() throws CardException, IOException, Iso7816UnsuccessfulExecutionException, PinBlockException {
        Log.i(TAG, "testWithRemoteCardServer()");

        RemoteCardChannel remoteCardChannel = new RemoteCardChannel("http://lnl07.lab.gematik.de:8083");
        HealthcardContext remoteCard = new HealthcardContext(remoteCardChannel, false);

        HealthcardContext egk;
        HealthcardContext hba;

        if (healthcardContext.isEGK()) {
            egk = healthcardContext;
            hba = remoteCard;
        } else {
            egk = remoteCard;
            hba = healthcardContext;
        }

        // test Card2Card authentication:

        // Use Case 1: Freischalten einer EGK durch einen HBA, z.B. zum Zugriff auf die Notfalldaten (NFD)
        // precondition: PIN.CH/PIN.SMC verified for HBA/SMC-B
        hba.verify("123456");
        egk.authenticateOtherCard(hba);

        // Use Case 2: Echtheitsprüfung einer EGK durch einen HBA
        hba.authenticateOtherCard(egk);

        // test access to NFD in order to verify the success of Card2Card authentication:

        //SELECT DF.HCA
        egk.selectADF("D27600000102");

        //SELECT DF.HCA/DF.NFD
        egk.selectADF("D27600014407");

        //SELECT EF.NFD
        egk.selectEF("D010");

        if (egk.isExtendedLengthApduSupported()) {
            egk.setMaxLength(4096); // readBinary funktioniert nur bis LE=4096, LE=8192 und größer liefert den Error 6F00
        }

        // Read length
        ShortBuffer shortBuffer = ByteBuffer.wrap(egk.read(0, 2)).asShortBuffer();

        // Read data
        byte[] data = egk.read(2, shortBuffer.get(0));
        String result = Utilities.unzip(data);
        Log.i(TAG, result);
    }

}
